from beach.beach_api import Beach
from beach.actor import Actor
from beach.actor import ActorHandle

import time

class BeachManager ( object ):

    def __init__( self, beach ):
        self._beach = beach


# This class is not meant to be imported/used manually by anyone other
# than the BeachManager object itself. It is the daemon loaded in the cloud
# to manage it from within.
class manager( Actor ):

    def init( self, parameters ):

        self.lastMasterHeartBeat = 0
        self.isMaster = False
        self.isMasterCooldown = False
        self.selfElectionTime = 0

        self.heartBeatFreq = 10
        self.masterTTL = self.heartBeatFreq * 2

        self.managers = self.getActorHandle( category = 'managers' )

        self.handle( 'heartbeat', self.recvMasterHeartBeat() )
        self.schedule( 10, self.checkElection )
        self.schedule( 5, self.masterAction )

        self.handle( '' )

    def deinit( self ):
        pass

    def recvMasterHeartBeat( self, msg ):
        remoteName = msg[ 'name' ]
        remoteTime = msg[ 'time' ]
        if remoteName != self.name:
            self.lastMasterHeartBeat = time.time()
            self.lastMasterHeartBeat = ( remoteName, remoteTime )

            if self.isMaster and remoteTime <= self.selfElectionTime:
                self.isMaster = False
                self.isMasterCooldown = False


    def checkElection( self ):
        if not self.isMaster and ( time.time() - self.masterTTL > self.lastMasterHeartBeat ):
            # Haven't heard from master in a while, need to elect
            self.isMasterCooldown = True
            self.selfElectionTime = time.time()
            self.isMaster = True
            self.lastMasterHeartBeat = 0

    def masterAction( self ):
        if not self.isMaster:
            return

        self.managers.broadcast( 'heartbeat', data = { 'name' : self.name, 'time' : self.selfElectionTime } )

        if self.isMasterCooldown:
            # Cooldown means we just elected ourselves as a Master, so we'll wait for a beacon time
            # to see if anyone is contesting the election.
            self.isMasterCooldown = False
            self.sleep( self.heartBeatFreq )

        if not self.isMaster:
            # Check if we received another higher priority master beacon.
            return

